@@ -8,7 +8,7 @@ package Debian::Debhelper::Buildsystem::cmake;
 
 use strict;
 use warnings;
-use Debian::Debhelper::Dh_Lib qw(%dh compat dpkg_architecture_value error is_cross_compiling);
+use Debian::Debhelper::Dh_Lib qw(%dh compat dpkg_architecture_value error is_cross_compiling get_buildoption);
 use parent qw(Debian::Debhelper::Buildsystem);
 
 my @STANDARD_CMAKE_FLAGS = qw(
@@ -17,6 +17,7 @@ my @STANDARD_CMAKE_FLAGS = qw(
   -DCMAKE_INSTALL_SYSCONFDIR=/etc
   -DCMAKE_INSTALL_LOCALSTATEDIR=/var
   -DCMAKE_EXPORT_NO_PACKAGE_REGISTRY=ON
+  -DCMAKE_FIND_USE_PACKAGE_REGISTRY=OFF
   -DCMAKE_FIND_PACKAGE_NO_PACKAGE_REGISTRY=ON
 );
 
@@ -83,19 +84,17 @@ sub configure {
 	my @flags = @STANDARD_CMAKE_FLAGS;
 	my $backend = $this->get_targetbuildsystem->NAME;
 
-	if (not compat(10)) {
-		push(@flags, '-DCMAKE_INSTALL_RUNSTATEDIR=/run');
-	}
-	if (not compat(12)) {
-		# Speed up installation phase a bit.
-		push(@flags, "-DCMAKE_SKIP_INSTALL_ALL_DEPENDENCY=ON");
-	}
+	push(@flags, '-DCMAKE_INSTALL_RUNSTATEDIR=/run') if not compat(10);
+	# Speed up installation phase a bit.
+	push(@flags, "-DCMAKE_SKIP_INSTALL_ALL_DEPENDENCY=ON") if not compat(12);
+	# Reproducibility #962474
+	push(@flags, "-DCMAKE_SKIP_RPATH=ON", '-DCMAKE_BUILD_RPATH_USE_ORIGIN=ON') if not compat(13);
 	if (exists($TARGET_BUILD_SYSTEM2CMAKE_GENERATOR{$backend})) {
 		my $generator = $TARGET_BUILD_SYSTEM2CMAKE_GENERATOR{$backend};
 		push(@flags, "-G${generator}");
 	}
 	if (not $dh{QUIET}) {
-		push(@flags, "-DCMAKE_VERBOSE_MAKEFILE=ON", "-DCMAKE_AUTOGEN_VERBOSE=ON");
+		push(@flags, "-DCMAKE_VERBOSE_MAKEFILE=ON");
 	}
 
 	if ($ENV{CC}) {
@@ -153,6 +152,16 @@ sub configure {
 	}
 }
 
+sub build {
+	my $this=shift;
+	my $target = $this->get_targetbuildsystem;
+	if ($target->NAME eq 'makefile') {
+		# Add VERBOSE=1 for #973029 when not asked to be quiet/terse.
+		push(@_, "VERBOSE=1") if not $dh{QUIET};
+	}
+	return $this->SUPER::build(@_);
+}
+
 sub test {
 	my $this=shift;
 	my $target = $this->get_targetbuildsystem;
@@ -162,7 +171,8 @@ sub test {
 		# -j1). So in order to simulate unlimited parallel, allow to fork a huge
 		# number of threads instead.
 		my $parallel = ($this->get_parallel() > 0) ? $this->get_parallel() : 999;
-		push(@_, "ARGS+=-j$parallel")
+		unshift(@_, "ARGS+=-j$parallel");
+		unshift(@_, "ARGS+=--verbose") if not get_buildoption("terse");
 	}
 	return $this->SUPER::test(@_);
 }
