@@ -14,7 +14,7 @@ use constant {
 	'SEQUENCE_TYPE_BOTH'                       => 'both',
 	'FLAG_OPT_SOURCE_BUILDS_NO_ARCH_PACKAGES'  => 0x1,
 	'FLAG_OPT_SOURCE_BUILDS_NO_INDEP_PACKAGES' => 0x2,
-	'UNSKIPPABLE_CLI_OPTIONS_BUILD_SYSTEM' => q(-S|--buildsystem|-D|--sourcedir(?:ectory)?|-B|--builddir(?:ectory)?),
+	'UNSKIPPABLE_CLI_OPTIONS_BUILD_SYSTEM' => q(-S|--buildsystem|-D|--sourcedir|--sourcedirectory|-B|--builddir|--builddirectory),
 };
 
 use Exporter qw(import);
@@ -81,7 +81,7 @@ sub sequence_type {
 	my ($sequence_name) = @_;
 	if ($sequence_name =~ m/-indep$/) {
 		return 'indep';
-	} elsif ($sequence_name =~ m/-arch/) {
+	} elsif ($sequence_name =~ m/-arch$/) {
 		return 'arch';
 	}
 	return 'both';
@@ -353,6 +353,7 @@ sub compute_selected_addons {
 	my $sequence_type = sequence_type($sequence_name);
 
 	my %addon_constraints = %{ Debian::Debhelper::Dh_Lib::bd_dh_sequences() };
+	my %explicitly_managed;
 
 	# Inject elf-tools early as other addons rely on their presence and it historically
 	# has been considered a part of the "core" sequence.
@@ -412,16 +413,48 @@ sub compute_selected_addons {
 	};
 
 	for my $request (@addon_requests) {
+		if ($request =~ m/^[+-]root[-_]sequence$/) {
+			error("Invalid request to skip the sequence \"root-sequence\": It cannot be disabled")
+				if $request =~ m/^-/;
+			error("Invalid request to load the sequence \"root-sequence\": Do not reference it directly");
+		}
 		if ($request =~ s/^[+]//) {
+			# Normalize "_" to "-" in the name.
+			$request =~ tr/_/-/;
 			$flush_disable_cache->() if %disabled_addons;
+			$explicitly_managed{$request} = 1;
 			push(@enabled_addons, $request) if not $enabled{$request}++;
 		} elsif ($request =~ s/^-//) {
+			# Normalize "_" to "-" in the name.
+			$request =~ tr/_/-/;
+			$explicitly_managed{$request} = 1;
 			$disabled_addons{$request} = 1;
 		} else {
 			error("Internal error: Invalid add-on request: $request (Missing +/- prefix)");
 		}
 	}
 
+	if (compat(14, 1) && getpackages() == 1 && !exists($explicitly_managed{'single-binary'})) {
+		if (not compat(13, 1)) {
+			warning("Implicitly activating single-binary dh addon for backwards compatibility.  In compat 14+,");
+			warning("this fallback will *not* happen automatically and dh_auto_install will instead use a");
+			warning("different default for --destdir, which can cause the source to produce an empty binary package");
+			warning();
+			warning('To keep the existing behaviour, please activate the single-binary addon explicitly.');
+			warning('This can be done by adding "dh-sequence-single-binary" to Build-Depends or passing');
+			warning('--with=single-binary to dh.');
+			warning();
+			warning('If you have solved this issue differently (e.g., by passing --destdir explicitly to');
+			warning('dh_auto_install or not using dh_auto_install at all) and want to silence this warning');
+			warning('without activating the addon, you can do that by passing --without=single-binary to dh');
+			warning('to explicitly acknowledge the change.');
+			warning();
+			warning('Please see the description of the "single-binary" in "man dh" for more details of what');
+			warning('it does and why this is changing from implicit behaviour to explicitly opt-in.');
+		}
+		push(@enabled_addons, 'single-binary');
+	}
+
 	$flush_disable_cache->() if %disabled_addons;
 	return map {
 		{
@@ -447,25 +480,30 @@ sub load_sequence_addon {
 
 sub check_for_obsolete_commands {
 	my ($full_sequence) = @_;
-	my ($found_obsolete_targets);
+	my ($found_obsolete_targets, $min_compat);
 	for my $command (@{$full_sequence}) {
 		if (exists($Debian::Debhelper::DH::SequenceState::obsolete_command{$command})) {
-			my $addon_name = $Debian::Debhelper::DH::SequenceState::obsolete_command{$command};
+			my $addon_name = $Debian::Debhelper::DH::SequenceState::obsolete_command{$command}[1];
 			error("The addon ${addon_name} claimed that $command was obsolete, but it is not!?");
 		}
 	}
 	for my $command (sort(keys(%Debian::Debhelper::DH::SequenceState::obsolete_command))) {
+		my ($addon_name, $error_compat) = @{$Debian::Debhelper::DH::SequenceState::obsolete_command{$command}};
+		$addon_name = 'debhelper' if $addon_name eq 'root-sequence';
 		for my $prefix (qw(execute_before_ execute_after_ override_)) {
 			for my $suffix ('', '-arch', '-indep') {
 				my $target = "${prefix}${command}${suffix}";
 				if (defined(rules_explicit_target($target))) {
 					$found_obsolete_targets = 1;
-					warning("The target ${target} references a now obsolete command and will not be run!");
+					$min_compat //= $error_compat;
+					$min_compat = $error_compat if $error_compat < $min_compat;
+					warning("The target ${target} references a now obsolete command and will not be run!"
+							. " (Marked by ${addon_name}, will be an error in compat $error_compat)");
 				}
 			}
 		}
 	}
-	if ($found_obsolete_targets and not compat(12)) {
+	if ($found_obsolete_targets and not compat($min_compat - 1)) {
 		error("Aborting due to left over override/hook targets for now removed commands.");
 	}
 	return;
@@ -658,7 +696,7 @@ sub run_through_command_sequence {
 		my @filtered_packages = _active_packages_for_command($command, $all_packages, $arch_packages, $indep_packages);
 
 		foreach my $package (@filtered_packages) {
-			if ($startpoint->{$package} > $i ||
+			if (($startpoint->{$package}//0) > $i ||
 				$logged->{$package}{$full_sequence->[$i]}) {
 				push(@opts, "-N$package");
 			}
@@ -715,7 +753,7 @@ sub run_through_command_sequence {
 				push(@cmd_options, @{$command_opts->{$command}})
 					if exists($command_opts->{$command});
 				push(@cmd_options, @opts);
-				run_sequence_command_and_exit_on_failure($command, @cmd_options);
+				run_sequence_command_and_exit_on_failure($command, _remove_dup_pkg_options(@cmd_options));
 			}
 		}
 
@@ -723,6 +761,25 @@ sub run_through_command_sequence {
 	}
 }
 
+sub _remove_dup_pkg_options {
+	my (@options) = @_;
+	my @filtered_options;
+	my $arch = 0;
+	my $indep = 0;
+	for my $option (@options) {
+		if ($option eq '-a' or $option eq '--arch') {
+			next if $arch;
+			$arch = 1;
+		}
+		if ($option eq '-i' or $option eq '--indep') {
+			next if $indep;
+			$indep = 1;
+		}
+		push(@filtered_options, $option);
+	}
+	return @filtered_options;
+}
+
 
 sub _stamp_target {
 	my ($command) = @_;
@@ -793,14 +850,17 @@ sub _stamp_target {
 				my $system = Debian::Debhelper::Dh_Buildsystems::load_buildsystem(undef, $need);
 				return 0 if defined($system);
 			} elsif ($type eq 'internal') {
-				if ($need ne 'bug#950723') {
+				if ($need eq 'bug#950723') {
+					$all_pkgs //= [ getpackages() ];
+					push(@{$all_pkgs}, map { "${_}@"} getpackages());
+					push(@packages, map { "${_}@"} @packages);
+				} elsif ($need eq 'rrr') {
+					my $req = Debian::Debhelper::Dh_Lib::root_requirements();
+					return 0 if $req ne 'none';
+				} else {
 					warning('Broken internal NOOP hint; should not happen unless someone is using implementation details');
 					error("Unknown internal NOOP type hint in ${command}: ${need}");
 				}
-
-				$all_pkgs //= [ getpackages() ];
-				push(@{$all_pkgs}, map { "${_}@"} getpackages());
-				push(@packages, map { "${_}@"} @packages);
 			} else {
 				# Unknown hint - make no assumptions
 				return 0;
